package org.jxul.framework.treetable;     


import org.jxul.framework.*;
import java.util.*;

/**
 * XulTreeModell is a TreeTableModel representing a hierarchical file 
 * system. Nodes in the XulTreeModell are FileNodes which, when they 
 * are directory nodes, cache their children to avoid repeatedly querying 
 * the real file system. 
 * 
 * @version %I% %G%
 *
 * @author Philip Milne
 * @author Scott Violet
 */

public class XulTreeModel extends AbstractTreeTableModel 
                             implements TreeTableModel {

    // Names of the columns.
    private Vector cNames;

    // The the returned file length for directories. 
    public static final Integer ZERO = new Integer(0); 

    public XulTreeModel(XulTreeHead th, XulTreeChildren tc) { 
	super(new XulTreeNode(tc));
	cNames = th.getColumnNames();
    }

    //
    // Some convenience methods. 
    //

    protected Object getTreeElement(Object node) {
	XulTreeNode treeNode = ((XulTreeNode)node);
	return (treeNode.getTreeElement());
    }

    protected Object[] getChildren(Object node) {
	XulTreeNode treeNode = ((XulTreeNode)node); 
	return treeNode.getChildren(); 
    }

    //
    // The TreeModel interface
    //

    public int getChildCount(Object node) { 
	Object[] children = getChildren(node); 
	XulTreeNode tn = (XulTreeNode)node;
	if (children == null)
	{
		return (0);
	} 
	else
	{
		return (children.length);
	}
    }

    public Object getChild(Object node, int i) { 
	return getChildren(node)[i]; 
    }

    // The superclass's implementation would work, but this is more efficient. 
    public boolean isLeaf(Object node) 
    { 
		
         return ((XulTreeNode)node).isLeaf();
    }

    //
    //  The TreeTableNode interface. 
    //

    public int getColumnCount() {
	return cNames.size();
    }

    public String getColumnName(int column) 
    {
	String cname = cNames.elementAt(column).toString();
	return (cname);
    }

    public Class getColumnClass(int column) {
	if (column == 0)
		return TreeTableModel.class;
	else
		return String.class;
    }
 
    public Object getValueAt(Object node, int column) 
    {
	
	Object te = getTreeElement(node);
	
	if (te instanceof XulTreeRow)
	{
		XulTreeRow tr = (XulTreeRow)te;
		
		return (tr.getRowValues().elementAt(column).toString());			
	}
	else if (te instanceof XulTreeItem)
	{
		if (column == 0)
		{
			XulTreeNode xtn = (XulTreeNode)node;
			return (xtn.toString());					
		}
		else 
		{
			return ("");
		}
	}
	else
	{
		return (new String(""));
	}
    }
}

/* A FileNode is a derivative of the File class - though we delegate to 
 * the File object rather than subclassing it. It is used to maintain a 
 * cache of a directory's children and therefore avoid repeated access 
 * to the underlying file system during rendering. 
 */
class XulTreeNode { 
    String m_nodename = "";
    Object[] children = null; 

    Object m_tree_element = null;

    public XulTreeNode(Object tr)
    {
	this.m_tree_element = (Object)tr;

	//important for viewing the first item
	//since it is handled by another class	
	if (tr instanceof XulTreeItem)
	{
		XulTreeItem xtr = (XulTreeItem)tr;
		if (xtr.isContainer())
		{
			m_nodename = xtr.getItemName();
		}
	}
	else if (tr instanceof XulTreeRow)
	{
		XulTreeRow xr = (XulTreeRow)tr;
		m_nodename = xr.getRowValues().elementAt(0).toString();
	}
	else
	{
		m_nodename = "";
	}

    }    

    public boolean isLeaf()
    {
	if (m_tree_element instanceof XulTreeRow)
		return true;
	else
		return false;
    }
    /**
     * Returns the the string to be used to display this leaf in the JTree.
     */
    public String toString() { 
	return (m_nodename);
    }

    public Object getTreeElement() 
    {
	return (m_tree_element);
    }

    /**
     * Loads the children, caching the results in the children ivar.
     */

    private Object[] getChildren(XulTreeItem ti)
    {
                if (ti.getTreeChildren() != null)
			getChildren(ti.getTreeChildren());

                else if (ti.getRows() != null && ti.getRows().size() != 0)
                {
                        children = new XulTreeNode[ti.getRows().size()];
                        for (int i = 0; i < ti.getRows().size(); i++)
			{
                                children[i] = new XulTreeNode((XulTreeRow)ti.getRows().elementAt(i));
			}
			
                }         
      
	return (children);
    }
    private Object[] getChildren(XulTreeChildren tc)
    {
	Vector treeItems = tc.getItems();
	if (treeItems != null && treeItems.size() == 1)
	{
		//should be only one at this point
		children = getChildren((XulTreeItem)treeItems.elementAt(0));
	}		
	else
	{
		children = new XulTreeNode[treeItems.size()];
		for (int i = 0; i < treeItems.size(); i++)
		{
			children[i] = new XulTreeNode((XulTreeItem)treeItems.elementAt(i));
		}
	}
	return (children);
    }

    protected Object[] getChildren() {

	if (children != null) {	
	    return children; 
	}

	if (m_tree_element instanceof XulTreeItem)
	{
		XulTreeItem ti = (XulTreeItem)m_tree_element;
		return (getChildren(ti));
	}
	else if (m_tree_element instanceof XulTreeChildren) 
        {
		//return 	
		XulTreeChildren tc = (XulTreeChildren)m_tree_element;
		return (getChildren(tc));
	}		
	
	return children; 
    }
}

