// XulJsEngine.java 
//
// Kevin T. Smith
//
// This class should do all the loading & unloading
// of functions, connecting things to the DOM, etc. 
// It is based on the GoF Singleton design pattern,
// so there is one instance available per VM.
//
// 

package org.jxul.js;

import java.io.*;
import java.util.*;

import org.jxul.framework.*;
import org.jxul.util.*;
import org.jxul.css.*;
import org.jxul.js.*;

import org.w3c.dom.*;
import org.mozilla.javascript.*;


public class XulJsEngine 
{
        public static final boolean debug;

        static
        {
         // this variation works in both applets and applications
         boolean temp = false;
         try
         {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
         } catch (Exception e)
           { }
         debug = temp;
        }


	private static XulJsEngine _instance = null;
	private Scriptable m_scope = null;
	private Context     m_contxt = null;
	private XulJsDocument m_jsdoc = null;
	private XulDom 		m_xuldom = null;

	private Vector m_filesToLoad = new Vector();
	private Vector m_scriptsToLoad = new Vector();

	//This should be the real constructor that
 	//does all the set up. TBD.	

	protected XulJsEngine() throws Exception
	{
		initializeContext();
	}

	/*
	 * initializeContext has to run for every thread. Unfortunately, Rhino gets the current context
         * from the current thread, so our main thread's context is not relevant when we want to
         * respond to pushbutton events & such
         */
        private void initializeContext() throws Exception
        {
		m_contxt = Context.getCurrentContext();
		if (m_contxt == null)
		{
			m_contxt = Context.enter();
		        // Set version to JavaScript1.2 so that we get object-literal style
                	// printing instead of "[object Object]"
                	m_contxt.setLanguageVersion(Context.VERSION_1_2);
                	m_scope = m_contxt.initStandardObjects(null);

                	if (debug) System.out.println("Returned : " + m_scope.getClass().getName());
                	if (debug) System.out.println("Classname: " + m_scope.getClassName());

                	//add a document object to the scope that will
                	//mirror the browser document object
                	ScriptableObject.defineClass(m_scope, XulJsDocument.class);
                	ScriptableObject.defineClass(m_scope, XulJsWindow.class);
                	ScriptableObject.defineClass(m_scope, XulJsElement.class);

                	// implicitly create a document object (and anything else we need)

                	m_contxt.evaluateString(m_scope, "document = new document();", "<createDocument>", 1, null);
                	m_contxt.evaluateString(m_scope, "window = new window();", "<createWindow>", 1, null);
		
			//load all data that was loaded by other threads	
			int i ;	
			for (i = 0; i < m_scriptsToLoad.size(); i++)
				loadScript((String)m_scriptsToLoad.elementAt(i));
			for (i = 0; i < m_filesToLoad.size(); i++)
				loadScriptFile((File)m_filesToLoad.elementAt(i));
			setXulDom(m_xuldom);
			loadJSFunctions();
		}
}        

	public boolean setXulDom(XulDom xulDom)
	{
		boolean retval = true;
		m_xuldom = xulDom;

		m_jsdoc = (XulJsDocument) m_scope.get("document", m_scope);
		if (m_jsdoc != null)
			m_jsdoc.setXulDom(m_xuldom);
		else
			retval = false;
		return (retval);
	}		
	public void loadScriptFile(File scriptfile) throws Exception
	{
		initializeContext();
		m_contxt.evaluateReader(m_scope, new FileReader(scriptfile),"<script>", 1, null);
		
		if (!m_filesToLoad.contains(scriptfile))
			m_filesToLoad.add(scriptfile);

		if (debug)
			System.out.println("script " + scriptfile + " added to the scope");
	
	}
	public void loadScript(String script) throws Exception
	{
		initializeContext();

		m_contxt.evaluateString(m_scope, script, "<script>", 1,null);	
	
		if (!m_scriptsToLoad.contains(script))
			m_scriptsToLoad.add(script);

		if (debug)
			System.out.println("script " + script + " added to the scope");
	}

	public void executeFunction(String fname) throws Exception
	{
		initializeContext();

                m_contxt.evaluateString(m_scope, fname, "<fn>", 1,null);

                if (debug)
                        System.out.println("function" + fname + " executed. ");
	}	
	public void executeFunction(String contents, String fname) throws Exception
	{
		initializeContext();

		m_contxt.evaluateString(m_scope, contents, fname, 1, null);
	}

	//for debugging
	public void showIds()
	{
                // Scope id's 
                Object [] ids = m_scope.getIds();
                if (ids != null)
                {
                    System.out.println("ids.length: " + ids.length);
                    for (int k=0; k < ids.length; k++)
                        System.out.println("Id[" + k + "]-" + ids[k]);
                }
                else
                    System.out.println("ids is null.");
	}

	public static XulJsEngine getInstance()
	{
		if (_instance == null)
		{
			try 
			{
				_instance = new XulJsEngine();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		return (_instance);
	}
	//Loads convenience functions 

        private void loadJSFunctions() throws Exception
        {
		ScriptableObject.defineClass(m_scope, XulJsFunctions.class);
		String alert = "function alert(msg) { var f = new XulJsFunctions(); f.alert(msg);} ";
		String confirm = "function confirm(msg) {var f = new XulJsFunctions(); return (f.confirm(msg));}";
		m_contxt.evaluateString(m_scope, alert, "<cmd>", 1,null);
		m_contxt.evaluateString(m_scope, confirm, "<cmd>", 1, null); 
        }
	public Scriptable getScope()
	{
		return (m_scope);
	}
	public Context getContext()
	{
		return (m_contxt);
	}
		
}
